"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var t = __importStar(require("@babel/types"));
var ast_builders_1 = require("../utils/ast-builders");
var css_builders_1 = require("../utils/css-builders");
var extractCssFromExpression = function (node, meta) {
    if (t.isIdentifier(node)) {
        var binding = meta.parentPath.scope.getBinding(node.name);
        if (binding &&
            t.isVariableDeclarator(binding.path.node) &&
            t.isExpression(binding.path.node.init)) {
            return css_builders_1.buildCss(binding.path.node.init, meta);
        }
    }
    if (t.isArrayExpression(node)) {
        var css_1 = '';
        var variables_1 = [];
        node.elements.forEach(function (element) {
            if (!element) {
                return;
            }
            var result = extractCssFromExpression(element, meta);
            css_1 += result.css;
            variables_1 = variables_1.concat(result.variables);
        });
        return { css: css_1, variables: variables_1 };
    }
    return css_builders_1.buildCss(node, meta);
};
var getJsxAttributeExpression = function (node) {
    if (t.isStringLiteral(node.value)) {
        return node.value;
    }
    if (t.isJSXExpressionContainer(node.value)) {
        return node.value.expression;
    }
    throw new Error();
};
/**
 * Takes a JSX opening element and then transforms any usage of `css` prop to a compiled component.
 *
 * `<div css={{}}>`
 *
 * @param path Babel path - expects to be a JSX opening element.
 * @param state Babel state - should house options and meta data used during the transformation.
 */
exports.visitCssPropPath = function (path, meta) {
    var cssPropIndex = -1;
    var cssProp = path.node.attributes.find(function (attr, index) {
        if (t.isJSXAttribute(attr) && attr.name.name === 'css') {
            cssPropIndex = index;
            return true;
        }
        return false;
    });
    if (!cssProp || !cssProp.value) {
        return;
    }
    // Remove css prop
    path.node.attributes.splice(cssPropIndex, 1);
    var cssOutput = extractCssFromExpression(getJsxAttributeExpression(cssProp), meta);
    path.parentPath.replaceWith(ast_builders_1.buildCompiledComponent(__assign(__assign({}, meta.state.opts), { cssOutput: cssOutput, node: path.parentPath.node })));
};
//# sourceMappingURL=index.js.map