"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var template_1 = __importDefault(require("@babel/template"));
var ast_builders_1 = require("./utils/ast-builders");
var css_prop_1 = require("./css-prop");
var styled_1 = require("./styled");
exports.default = helper_plugin_utils_1.declare(function (api) {
    api.assertVersion(7);
    return {
        inherits: require('babel-plugin-syntax-jsx'),
        visitor: {
            Program: {
                exit: function (path, state) {
                    if (state.compiledImportFound && !path.scope.getBinding('React')) {
                        // React is missing - add it in at the last moment!
                        path.unshiftContainer('body', template_1.default.ast("import * as React from 'react'"));
                    }
                },
            },
            ImportDeclaration: function (path, state) {
                if (path.node.source.value === '@compiled/core') {
                    state.compiledImportFound = true;
                    path.node.specifiers = path.node.specifiers
                        .filter(function (specifier) {
                        return specifier.local.name !== 'styled' && specifier.local.name !== 'ClassNames';
                    })
                        .concat([ast_builders_1.importSpecifier('CC'), ast_builders_1.importSpecifier('CS')]);
                }
            },
            TaggedTemplateExpression: function (path, state) {
                if (!state.compiledImportFound) {
                    return;
                }
                styled_1.visitStyledPath(path, { state: state, parentPath: path });
            },
            CallExpression: function (path, state) {
                if (!state.compiledImportFound) {
                    return;
                }
                styled_1.visitStyledPath(path, { state: state, parentPath: path });
            },
            JSXOpeningElement: function (path, state) {
                if (!state.compiledImportFound) {
                    return;
                }
                css_prop_1.visitCssPropPath(path, { state: state, parentPath: path });
            },
        },
    };
});
//# sourceMappingURL=index.js.map